#
#   PROBE - Intro screen
#

from albow.widget import Widget
from albow.resource import get_image
from settings import intro_frame_time

class IntroScreen(Widget):

	num_frames = 3
	frame_display_time = 5000 # ms
	fade_time = 1000 # ms

	def __init__(self, shell):
		Widget.__init__(self, shell.rect)
		self.shell = shell
		self.show_frame(0)
	
	def show_frame(self, n):
		self.current_frame = n
		self.time = 0
		self.current_image = get_image("intro%d.jpg" % n, noalpha = True)
	
	def next_frame(self):
		n = self.current_frame + 1
		if n < self.num_frames:
			self.show_frame(n)
		else:
			self.shell.end_intro_screen()
	
	def draw(self, surf):
		image = self.current_image
		alpha = min(255,
			255 * self.time / self.fade_time)
		if self.current_frame + 1 < self.num_frames:
			alpha = min(alpha,
				255 * (self.frame_display_time - self.time) / self.fade_time)
		alpha = max(0, alpha)
		#print "IntroScreen.draw: alpha =", alpha
		image.set_alpha(alpha)
		frame = surf.get_rect()
		r = image.get_rect()
		r.center = frame.center
		surf.blit(image, r.topleft)
	
	def begin_frame(self):
		#print "IntroScreen.begin_frame" ###
		self.time += intro_frame_time
		if self.time >= self.frame_display_time:
			self.next_frame()

	def key_down(self, e):
		self.next_frame()

	def mouse_down(self, e):
		self.next_frame()
