#
#   PROBE - Name generation
#

from random import choice, uniform

initial_consonants = \
	"b,c,d,f,g,h,j,k,l,m,n,p,q,r,s,t,v,w,x,y,z".split(",") * 10 + \
	"bh,bl,br,bw,ch,cl,cr,dr,fl,fr,gh,gl,gr,kl,kr," \
	"pr,ps,q,sh,sk,sl,sm,sn,sp,sq,sr,st,sw," \
	"th,tr,ts,tw,vl,wh".split(",")

final_consonants = \
	"d,h,l,n,p,r,t,x,y,z".split(",") * 5 + \
	"gh,sh,sk,sp,st,th,ts".split(",")

other_consonants = \
	"bc,bd,bf,bg,bk,bp,bq,bs,bt,bv,bw,bz," \
	"ck,cq,ct," \
	"ckb,ckd,ckf,ckg,ckh,ckl,ckm,ckn,ckp,ckq,ckr,cks,ckt,ckv,ckw,ckz," \
	"df,dg,dh,dk,dl,dm,dn,dp,dq,ds,dt,dv,dw,dz," \
	"fb,fc,fd,fg,fh,fk,fm,fn,fp,fq,fs,ft,fw,fz," \
	"gb,gd,gf,gk,gm,gn,gp,gq,gs,gt,gv,gw,gz," \
	"kb,kd,kf,kg,kh,kl,km,kn,kp,kq,kr,ks,kt,kv,kw,kz," \
	"lb,lc,ld,lf,lg,lh,lk,lm,ln,lp,lq,lr,ls,lt,lv,lw,lz," \
	"mb,mc,md,mf,mg,mh,mk,ml,mn,mp,mq,mr,ms,mt,mv,mw,mz," \
	"nb,nc,nd,nf,ng,nh,nk,nl,nm,np,nq,nr,ns,nt,nv,nw,nz," \
	"pb,pc,pd,pf,pg,ph,pk,pl,pm,pn,pq,pt,pv,pw,pz," \
	"rb,rc,rd,rf,rg,rh,rk,rl,rm,rn,rp,rq,rs,rt,rv,rw,rz," \
	"sb,sc,sd,sf,sg,sv,sz," \
	"tb,tc,td,tf,tg,tk,tl,tm,tn,tp,tq,tv,tz," \
	"vb,vc,vd,vf,vg,vh,vk,vm,vn,vp,vq,vw,vz," \
	"wb,wc,wd,wf,wg,wk,wl,wm,wn,wp,wq,wr,ws,wt,wv,wz," \
	"xb,xc,xd,xf,xg,xh,xk,xl,xm,xn,xp,xq,xr,xt,xv,xw," \
	"yb,yc,yd,yf,yg,yh,yk,yl,ym,yn,yp,yq,yr,ys,yt,yv,yw,yz," \
	"zb,zc,zd,zf,zg,zh,zk,zl,zm,zn,zp,zq,zr,zs,zt,zv,zw".split(",")

initial_vowels = "eeeeeaaaaiiioou"
vowels = initial_vowels
final_vowels = vowels + "y" * 5

doubleable_letters = "abcdefglmnoprstvz"

substitutions = [
	("yy", "ey"),
	("iy", "ay"),
	("uw", "aw"),
	("iw", "ow"),
	("uy", "oy"),
	("q",  "qu"),
	("quu", "quo"),
]

# States
INITIAL_VOWEL = 0
INITIAL_CONSONANT = 1
VOWEL = 2
CONSONANT = 3
FINAL_VOWEL = 4
FINAL_CONSONANT = 5

initial_states = INITIAL_VOWEL, INITIAL_CONSONANT

state_choices = {
	INITIAL_VOWEL:     initial_vowels,
	INITIAL_CONSONANT: initial_consonants,
	VOWEL:             vowels,
	CONSONANT:         initial_consonants + other_consonants,
	FINAL_VOWEL:       final_vowels,
	FINAL_CONSONANT:   final_consonants,
}

next_states = {
	INITIAL_VOWEL:     ([CONSONANT], [FINAL_CONSONANT]),
	INITIAL_CONSONANT: ([VOWEL],     [FINAL_VOWEL]),
	VOWEL:             ([CONSONANT], [FINAL_CONSONANT]),
	CONSONANT:         ([VOWEL],     [FINAL_VOWEL]),
	FINAL_VOWEL:       final_vowels,
	FINAL_CONSONANT:   final_consonants,
}

def chance(p):
	return uniform(0, 1) < p

def random_word(num_parts):
	parts = []
	last_part = num_parts - 1
	state = choice(initial_states)
	last_part_was_vowel = False
	for i in xrange(num_parts):
		part = choice(state_choices[state])
		if state <> INITIAL_CONSONANT \
				and len(part) <= 2 \
				and part[0] in doubleable_letters \
				and not (last_part_was_vowel and len(parts[-1]) > 1) \
				and chance(0.1):
			part = part[0] + part
		parts.append(part)
		last_part_was_vowel = state in (INITIAL_VOWEL, VOWEL, FINAL_VOWEL)
		if i + 1 < num_parts:
			last = i + 1 == last_part
			state = choice(next_states[state][last])
	s = "".join(parts)
	for r in substitutions:
		s = s.replace(*r)
	return s.capitalize()
