#
#   PROBE - Text formatting utilities
#

from textwrap import wrap

class TextWriter(object):

	def __init__(self, surface, font, color, x, y, tabs = ()):
		self.surface = surface
		self.font = font
		self.color = color
		self.left_margin = x
		self.x = x
		self.y = y
		self.tabs = tabs
	
	def write(self, text, font = None):
		font = font or self.font
		buf = font.render(text, True, self.color)
		self.surface.blit(buf, (self.x, self.y))
		self.x += buf.get_width()
	
	def writeln(self, text, font = None):
		self.write(text, font)
		self.newline()
	
	def newline(self):
		self.x = self.left_margin
		self.y += self.font.get_linesize()
	
	def write_lines(self, text):
		for line in text.split("\n"):
			self.writeln(line)

	def write_paragraph(self, text, width):
		for line in wrap(text, width):
			self.writeln(line)
	
	def write_paragraphs(self, text, width):
		for para in text.split("\n\n"):
			self.write_paragraph(para, width)

	def vspace(self, h):
		self.y += h

	def tab(self, i):
		if i <= 0:
			t = 0
		else:
			t = self.tabs[i - 1]
		self.x = max(self.x, t)
